KILL = false;  // this needs to be initialized first


/////   Thumbnail Object
// ==========================
// - encompasses a canvas & server-side Evograph
// - is reusable
// 
Thumbnail = function(x, y){
	this.xpos = x;
	this.ypos = y;
	this.evo = 0;   // make canvas with random ID
	this.canvasID = Math.random().toString(36).substr(2, 5);
	this.canvas = document.createElement('canvas');
	this.canvas.id     = this.canvasID;
	this.canvas.width  = DIM;
	this.canvas.height = DIM;
	this.canvas.className = "thumbNail";
	// inject canvas into html & set its location
	document.body.appendChild(this.canvas);
	this.updateCanvas();
	this.killed = false;
}

// move associated canvas to (xpos, ypos)
Thumbnail.prototype.updateCanvas = function(){
	var elem = document.getElementById(this.canvasID);
	elem.style.position = "absolute";
	elem.style.left =     ""+this.xpos+"px";
	elem.style.top  =     ""+this.ypos+"px";
		if(mouseX > this.xpos)
			if(mouseX < this.xpos + DIM)
				if(mouseY > this.ypos)
					if(mouseY < this.ypos + DIM)
					    if(this.evo != 0)
					    	if(!this.killed){
					    		
								if(KILL){  // if user has clicked & mouse is hovering...
									this.killed = true;  // send server a message to kill associated Evo
									KILL = false;		 // & clear canvas for responsive effect
									var ctxt = this.canvas.getContext('2d');
									ctxt.clearRect(0, 0, DIM, DIM);
									display.kills++;
									Meteor.call('killEvo', this.evo.index, this.evo.password, function(e,r){    							
									});
								} else if(STATE.save){
									STATE.save = false;
									saveCanvas(this.canvas);
								} else if(STATE.expand){
									STATE.expand = false;
									expandThumb(this.evo.index);
								} else if(STATE.copy){
									var eqs;
									Meteor.call('getEquation', this.evo.index, function(e,r){ 
										eqs = r;   	
										window.prompt ("The thumbnail you just clicked was generated pixel-by-pixel by these functions with x and y being normalized coordinates (0-1).", eqs);						
									});
								  	STATE.copy = false;
								}
							}
}

// move vertically
Thumbnail.prototype.shift = function(amount){
	this.ypos += amount;
	this.updateCanvas();
}

// set equal to another thumbnail
Thumbnail.prototype.setFrom = function(oth){
	this.xpos = oth.xpos;
	this.ypos = oth.ypos;
	this.canvas = oth.canvas;
	this.canvasID = oth.canvasID;
	this.evo = oth.evo;
}

// load random evo from server, place its image on canvas
Thumbnail.prototype.getImage = function(){
	var t = this;
	Meteor.call('getThumbnail', function(e,r){
		t.evo = r;
		t.placeImage();
		this.killed = false;
	});
}

// places image from server onto thumbnail canvas
Thumbnail.prototype.placeImage = function(){
	if(this.evo === 0) return;
	var elem = document.getElementById(this.canvasID);
	var ctx = elem.getContext('2d');
	var imageData = ctx.getImageData(0, 0, DIM, DIM);
	var data = imageData.data;
	data.set(this.evo.pixels);
	ctx.putImageData(imageData, 0, 0);
}




/////   Row Object
// ==================
// - horizontal group of thumbnails
// - makes monitoring buffer easier
// 
Row = function(num, y){
	this.pop = num;
	this.thumbs = [];
	this.ypos = y;
	// make sure everything's centered
	var start = (window.innerWidth - num*DIM)/2 - DIM;
	while(num > 0){
		this.thumbs.push(new Thumbnail(start + num*DIM, y));
		num--;
	}
}

// move entire row vertically
Row.prototype.upshift = function(shif){
	this.ypos += shif;
	for(var i = 0; i < this.pop; i++)
		this.thumbs[i].shift(shif);
}

// delegate 'getImage' to each thumbnail
Row.prototype.getImages = function(){
	for(var i = 0; i < this.pop; i++)
		this.thumbs[i].getImage(0);
}






/////   Population Object
// =========================
// - encompasses an array of rows of thumbnails
// - persistently displays all individuals
// - handles meteor server calls, ect.
//
Population = function(x, y, d){
	this.dim  = d; // sidelength of thumbnails
	this.dimx = x; // thumbs per row
	this.dimy = y; // # of row
	this.rows = []; // row objects
	// compose row array
	for(var i = 0; i < y; i++)
		this.rows.push( new Row(x, i*DIM));
	// impose persistant flow via 'setInterval'
	var t = this; //* we need 'this' within the clock
	this.clock = self.setInterval(function(){
		t.flow(-1);
	},8);
}

// slide entire population buffer upwards
Population.prototype.flow = function(amount){
	for(var i = 0; i < this.dimy; i++)   	// move all rows upward
		this.rows[i].upshift(amount);
		 	 	 	 	 	 	 	 	 	// if row is above top of screen, append it to buffer
	if(this.rows[0].ypos < -DIM){
		this.rows.push(new Row(this.dimx, Math.max(window.innerHeight+DIM, this.rows[this.dimy-1].ypos + DIM)));
		this.rows.splice(0,1);
		this.rows[this.dimy - 1].getImages();
	}
}