/////   Evo Object
// ==================
// - includes a genome for each color (r,g,b)
// - facilitates image generation
//
Evo = function(ind){
	this.generateTrees(4); // initialize trees
	this.getEQs();    // load their respective equations
	this.loadImage(); // store image data
	this.index = ind; // record respective index
	// create unique password
	this.unique = Math.random().toString(36).substr(2, 8);
}

// generates random trees
Evo.prototype.generateTrees = function(depth){
	this.rgen = new Node(5);
	this.ggen = new Node(5);
	this.bgen = new Node(5);
}

// loads tree equations into string form
// - later will be interpreted via 'eval()'
Evo.prototype.getEQs = function(){
	this.redEQ = this.rgen.getEQ();
	this.greenEQ = this.ggen.getEQ();
	this.blueEQ = this.bgen.getEQ();
}

// sets a pixel within a Uint8Array
Evo.prototype.setPixel = function(arr, x,y,r,g,b,a, wid){
	if(arguments.length < 8) wid = DIM;
	var i = (x + y * wid) * 4;
	arr[i+0] = r;
	arr[i+1] = g;
	arr[i+2] = b;
	arr[i+3] = a;
}

// computes the contents of a DIMxDIM image
// - utilized 'eval()' with the EQs (which are now strings)
Evo.prototype.loadImage = function(){
	this.imagePixels = new Uint8Array(4*DIM*DIM);
	for(var i = 0; i < DIM; i++){
		for(var j = 0; j < DIM; j++){
			var x = 255*i/DIM;
			var y = 255*j/DIM;
			this.setPixel(this.imagePixels,
					 	  i, j, eval(this.redEQ)
					 	      , eval(this.greenEQ)
					 	      , eval(this.blueEQ)
					 	      , 255);
		}
	}
}

// generate an object to send to client
Evo.prototype.forThumb = function(){
	// requesting a thumbnail
	var obj = {};
	obj.pixels = this.imagePixels;
	obj.index = this.index;
	obj.password = this.unique;
	return obj;
}

// simulate the death of an individual
// - replace with new offspring
Evo.prototype.attemptKill = function(pass){
	if(pass == this.unique){      // replace each genome
		this.rgen = descendent();
		this.ggen = descendent();
		this.bgen = descendent();
		this.getEQs();    // load new equations
		this.loadImage(); // load new image & unique password
		this.unique = Math.random().toString(36).substr(2, 8);
		return true;
	}
	return false;
}


Evo.prototype.getFull = function(len){
	var pix = new Uint8Array(4*len*len);
	for(var i = 0; i < len; i++){
		for(var j = 0; j < len; j++){
			var x = 255*i/len;
			var y = 255*j/len;
			this.setPixel(pix,
					 	  i, j, eval(this.redEQ)
					 	      , eval(this.greenEQ)
					 	      , eval(this.blueEQ)
					 	      , 255
					 	      , len);
		}
	}
	return pix;
}

Evo.prototype.getFormattedEQs = function(){
	return "red: " + this.redEQ + "\n\ngreen: " + this.greenEQ + "\n\nblue: " + this.blueEQ;
}