/////   Leaf Object
// ===================
// - provides an input to tree
// - can be a number or variable (x or y)
Leaf = function(){
	this.val = Math.random()*2.0;
	this.from = Math.round(Math.random()*234234)%3;
}

// return value based on previously determined source
Leaf.prototype.getVal = function(x, y){
	switch(this.from){
		case 0:   return x;
		case 1:   return y;
		default:  return this.val;
	}
}

// put equation into string form
// - recursive base case
Leaf.prototype.getEQ = function(){
	switch(this.from){
		case 0:   return 'x';
		case 1:   return 'y';
		default:  return this.val;
	}
}

// alter value slightly
Leaf.prototype.vary = function(){
	this.val += Math.random()*.4 - .2;
}

// make probabilistic mutations 
Leaf.prototype.pointMutate = function(prob){
	if(Math.random() < prob)
		this.val = Math.random()*2.0;
	if(Math.random() < prob)
		this.from = Math.round(Math.random()*234234)%3;
}

// return a copy
Leaf.prototype.clone = function(prob){
	var newLeaf = new Leaf();
	newLeaf.val = this.val;
	newLeaf.from = this.from;
	return newLeaf;
}

// placeholders...
Leaf.prototype.findChildren = function(curr){
	return [curr];}
Leaf.prototype.getChild = function(dirs){
	return this;}
Leaf.prototype.setChild = function(oth){}








/////   Node Object
// ====================
// - is a 'split in the genomic tree
// - sprouts two leaves/nodes
// - value is based on op, and child leaf outputs
// - argument limits initial depth
//
Node = function(numDeep){
	Leaf.call(this); // inherits from leaf
	this.op = Math.round(Math.random()*234234)%6;
	if(numDeep > 2){
		this.c1 = new Node(numDeep-1);
		this.c2 = new Node(numDeep-1);
	} else{
		this.c1 = new Leaf();
		this.c2 = new Leaf();
	}
}
// inherit...
Node.prototype = new Leaf();
	
// return value according to op and child vals
Node.prototype.getVal = function(x,y){
	switch(this.op){
		case 0:   return this.c1.getVal(x,y) + this.c2.getVal(x,y);
		case 1:   return this.c1.getVal(x,y) - this.c2.getVal(x,y);
		case 2:   return this.c1.getVal(x,y) * this.c2.getVal(x,y);
		case 3:   return this.c1.getVal(x,y) / this.c2.getVal(x,y);
		case 4:   return this.c1.getVal(x,y) % this.c2.getVal(x,y);
		case 5:   return this.c1.getVal(x,y) ^ this.c2.getVal(x,y);
		default:  return 1;
	}
}

// recursively generate string equation
Node.prototype.getEQ = function(){
	var out = "(";
	if(this.op != 5){
		out += this.c1.getEQ();
		switch(this.op){
			case 0:   out += ' + ';   break;
			case 1:   out += ' - ';   break;
			case 2:   out += ' * ';   break;
			case 3:   out += ' / ';   break;
			case 4:   out += ' % ';   break;
			default:  return "?";
		}
		out += this.c2.getEQ();
	} else{
		out += "Math.pow(" + this.c1.getEQ() + "," + this.c2.getEQ() + ")";
	}
	out += ")";
	return out;
}

// recursively vary
Node.prototype.vary = function(){
	this.c1.vary();
	this.c2.vary();
}

// recursively mutate, and mutate self
Node.prototype.pointMutate = function(prob){
	if(Math.random() < prob)
		this.op = Math.round(Math.random()*234234)%6;
	this.c1.pointMutate(prob);
	this.c2.pointMutate(prob);
}

// grow/contract tree branches
// - positive input for expansion, negative for contraction
Node.prototype.morph = function(prob){
	// if expanding
	if(prob > 0){
		// first child
		if(!(this.c1 instanceof Node))
			if(0 < prob/2)
				this.c1 = new Node(2);
		else
			this.c1.morph(prob);
		// second child
		if(!(this.c2 instanceof Node))
			if(Math.random() < prob/2)
				this.c2 = new Node(2);
		else
			this.c2.morph(prob);
	}
	// if contracting
	else{
		// first child
		if((this.c1.c1 instanceof Node)  ||  (this.c1.c2 instanceof Node))
			this.c1.morph(prob);
		else
			if(Math.random() < -prob) this.c1 = new Leaf();
		// second child
		if(this.c2.c1 instanceof Node  ||  this.c2.c2 instanceof Node)
			this.c2.morph(prob);
		else
			if(Math.random() < -prob) this.c2 = new Leaf();
	}
}

// recursively clone object
Node.prototype.clone = function(){
	var newNode = new Node(1);
	newNode.c1 = this.c1.clone();
	newNode.c2 = this.c2.clone();
	newNode.op = this.op;
	return newNode;
}

// compile an array of binary decision treess
Node.prototype.findChildren = function(curr){
	if(arguments.length === 0)
		curr = [];
	var curr1 = curr.slice(0);
	var curr2 = curr.slice(0);
	curr1.push(false);
	curr2.push(true);
	var all = [curr];
	return all.concat(this.c1.findChildren(curr1), this.c2.findChildren(curr2));
}

// return node given by binary decision tree, dirs
Node.prototype.getChild = function(dirs){
	if(arguments.length == 0) dirs = [];
	if(dirs.length == 0) 	  return this;
	if(dirs[0] == true){
		dirs.splice(0,1);
		return this.c2.getChild(dirs);
	} else{
		dirs.splice(0,1);
		return this.c1.getChild(dirs);
	}
}

// set child which can be found by dirs (binary decision tree)
Node.prototype.setChild = function(oth, dirs){
	if(arguments.length < 2) dirs = [];
	if(dirs.length <= 1){
		if(dirs[0] == true)
			this.c2 = oth.clone();
		else
			this.c1 = oth.clone();
		return;
	}
	
	if(dirs[0] == true){
		dirs.splice(0,1);
		this.c2.setChild(oth, dirs);
	} else{
		dirs.splice(0,1);
		this.c1.setChild(oth, dirs);
	}
}

// cross breed with other Node
Node.prototype.cross = function(oth){
	var newNode = this.clone();
	var newChildren = newNode.findChildren();
	var othChildren = oth.findChildren();
	for(var i = 0; i < newChildren.length; i++){
		var ind = Math.round(Math.random()*(newChildren.length-1));
		for(var j = 0; j < othChildren.length; j++)
			if(areEqual(newChildren[ind], othChildren[j])){
				newNode.setChild(oth.getChild(othChildren[ind]), newChildren[ind]);
				return newNode;
			}
	}
	console.log("performed unsuccessful cross operation");
	return newNode;
}

// simple array comparison
areEqual = function(arr1, arr2){
	if(arr1.length != arr2.length) return false;
	for(var i = 0; i < arr1.length; i++){
		if(arr1[i] != arr2[i]) return false;
	}
	return true;
}
